#include "cubewidget.h"

CubeWidget::CubeWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{

}

QSize CubeWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize CubeWidget::sizeHint() const
{
    return QSize(600, 400);
}

void CubeWidget::initializeGL ()
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);

    tesselation = 0;
    // enable default shading
    home();
    showFlat();

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    static GLfloat lightPosition[4] = { 0.5, 0.0, 2.0, 1.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
}

void CubeWidget::setMaterial(GLfloat *color )
{
    glMaterialfv(GL_FRONT,GL_DIFFUSE,color);
    glMaterialfv(GL_FRONT,GL_SPECULAR,color);
    glMaterialfv(GL_FRONT,GL_SHININESS,color);
}

void CubeWidget::paintGL ( )
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();

    glTranslatef(0.0f,0.0f,-zoom);

    QMatrix4x4 mat = QMatrix4x4();
    mat.rotate(*rotation);
    glMultMatrixf(mat.data());




    GLfloat red[] = {1.0,0.0,0.0};
    GLfloat green[] = {0.0,1.0,0.0};
    GLfloat blue[] = {0.0,0.0,1.0};
    GLfloat cyan[] = {0.0,1.0,1.0};
    GLfloat magenta[] = {1.0,0.0,1.0};
    GLfloat yellow[] = {1.0,1.0,0.0};

    glBegin(GL_QUADS);
    float increment = 1.0/(pow(2,tesselation));

    setMaterial(red);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment, 0.5f,y);
            glVertex3f(x, 0.5f,y);
            glVertex3f(x, 0.5f, y+increment);
            glVertex3f( x+increment, 0.5f, y+increment);
        }
    }

    setMaterial(magenta);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment, -0.5f,y+increment);
            glVertex3f(x, -0.5f,y+increment);
            glVertex3f(x, -0.5f, y);
            glVertex3f( x+increment, -0.5f, y);
        }
    }

    setMaterial(green);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment,y+increment, 0.5f);
            glVertex3f(x, y+increment, 0.5f);
            glVertex3f(x,  y, 0.5f);
            glVertex3f( x+increment, y, 0.5f);
        }
    }

    setMaterial(yellow);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment,y, -0.5f);
            glVertex3f(x, y,-0.5f);
            glVertex3f(x, y+increment,-0.5f);
            glVertex3f( x+increment,y+increment, -0.5f);
        }
    }


    setMaterial(blue);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( -0.5f, x+increment,y+increment);
            glVertex3f( -0.5f,x+increment,y);
            glVertex3f( -0.5f,x, y);
            glVertex3f(  -0.5f,x, y+increment);
        }
    }

    setMaterial(cyan);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( 0.5f, x+increment,y);
            glVertex3f( 0.5f,x+increment,y+increment);
            glVertex3f( 0.5f,x, y+increment);
            glVertex3f(  0.5f,x, y);
        }
    }
    glEnd();
}



void CubeWidget::resizeGL(int width , int height )
{
    glMatrixMode(GL_PROJECTION);

    glViewport(0, 0, width, height);

    glLoadIdentity();
    perspective(45.0,1.0*width/height,0.01,100.0);

    glMatrixMode(GL_MODELVIEW);
}

void CubeWidget::perspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
{
    GLdouble xmin, xmax, ymin, ymax;

    ymax = zNear * tan( fovy * M_PI / 360.0 );
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;

    glFrustum( xmin, xmax, ymin, ymax, zNear, zFar );
}


void CubeWidget::showWireframe()
{

    qDebug("show Wireframe");
    glShadeModel(GL_FLAT);
   // glDisable(GL_CULL_FACE);

    glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
    updateGL();
}

void CubeWidget::showFlat()
{
    qDebug("show Flat");
    glShadeModel(GL_FLAT);
   // glEnable(GL_CULL_FACE);
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    updateGL();
}

void CubeWidget::showGouraut()
{
    qDebug("show Gouraut");
    glShadeModel(GL_SMOOTH);
  //  glEnable(GL_CULL_FACE);
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    updateGL();
}

void CubeWidget::showPhong()
{
    qDebug("show Phong");
    showFlat();

}

void CubeWidget::setTessellation(int t)
{
    tesselation = t;
    updateGL();
    qDebug()<<"Tesselation" << t;
}

void CubeWidget::home()
{
    zoom = 3;
    rotation = new QQuaternion();
    updateGL();
}

void CubeWidget::mousePressEvent(QMouseEvent *event )
{

   // if (event->isAccepted())
   //     return;

    if (event->buttons() & Qt::LeftButton) {
        qDebug("press");
        lastPos = trackballPoint(event->pos().x(),event->pos().y());


        event->accept();
    } else {
    }

}

void CubeWidget::mouseMoveEvent(QMouseEvent *event )
{

    //http://web.cse.ohio-state.edu/~hwshen/781/Site/Slides_files/trackball.pdf
    //http://doc.qt.io/qt-5/qtwidgets-graphicsview-boxes-example.html

   // if (event->isAccepted())
    //    return;

    if (event->buttons() & Qt::LeftButton) {
         qDebug("move");
        move(trackballPoint(event->pos().x(),event->pos().y()));

        //m_trackBalls[0].move(pixelPosToViewPos(event->scenePos()), m_trackBalls[2].rotation().conjugate());
        event->accept();
    } else {
        //m_trackBalls[0].release(pixelPosToViewPos(event->scenePos()), m_trackBalls[2].rotation().conjugate());
    }
}

void CubeWidget::move(QVector3D *newPos )
{
     QVector3D axis = QVector3D::crossProduct(*lastPos,*newPos);

     //warum so besser
     float angle = 180 / M_PI * std::asin(std::sqrt(QVector3D::dotProduct(axis, axis)));
     axis.normalize();

     axis = rotation->conjugate().rotatedVector(axis);
     QQuaternion newRot = QQuaternion::fromAxisAndAngle(axis, angle) * *rotation;
     rotation = new QQuaternion(newRot.toVector4D());

     lastPos = newPos;
     updateGL();

}

QVector3D* CubeWidget::trackballPoint(int x, int y){
    float xo,yo,zo;

    qDebug()<<"x:"<< x << " y:"<<y;
    xo = ((2.0*x)-width())/ height();
    yo = (height()-(2.0*y))/width();

    float d = sqrt(xo*xo+yo*yo);
    zo = qMax(qCos(M_PI_2*d),qreal(0.0)); //qMin(d,1.0f)



    QVector3D *pos = new QVector3D(xo,yo,zo);
    pos->normalize();
    qDebug()<<"x:"<< xo << " y:"<<yo<<" z:"<<zo;
    return pos;
}

void CubeWidget::wheelEvent(QWheelEvent *event )
{
    if(event->delta()<0)
        zoom++;
    else
        zoom--;

    updateGL();
}
